Package: ifaTools
Date: 2022-01-19
Title: Toolkit for Item Factor Analysis with 'OpenMx'
Authors@R: c(person(c("Joshua","N."), family="Pritikin",
    email="jpritikin@pobox.com", role=c('cre','aut')))
Version: 0.23
Description: Tools, tutorials, and demos of Item Factor Analysis using 'OpenMx'.
    This software is described in Pritikin & Falk (2020) <doi:10.1177/0146621620929431>.
License: AGPL (>= 3)
VignetteBuilder: knitr
Imports: methods
Suggests: testthat, roxygen2, knitr (>= 1.8), gridExtra, plyr, xtable
Depends: shiny (>= 0.10), ggplot2, reshape2, rpf (>= 0.48), OpenMx (>=
        2.3.1), R (>= 2.14.0)
URL: https://github.com/jpritikin/ifaTools
RoxygenNote: 7.1.1
NeedsCompilation: no
Packaged: 2022-01-19 21:06:57 UTC; joshua
Author: Joshua N. Pritikin [cre, aut]
Maintainer: Joshua N. Pritikin <jpritikin@pobox.com>
Repository: CRAN
Date/Publication: 2022-01-19 22:12:42 UTC
Built: R 4.6.0; ; 2026-04-26 13:34:40 UTC; unix
